@@ -1,34 +1,52 @@
 use crate::job_scheduler::JobsSchedulerLocked;
+use crate::JobScheduler;
 use chrono::{DateTime, Utc};
 use cron::Schedule;
+use std::collections::HashMap;
+use std::future::Future;
+use std::pin::Pin;
 use std::str::FromStr;
 use std::sync::{Arc, RwLock};
-use uuid::Uuid;
-use crate::JobScheduler;
 use std::time::Duration;
 use tokio::task::JoinHandle;
-use std::pin::Pin;
-use std::future::Future;
+use uuid::Uuid;
 
 pub type JobToRun = dyn FnMut(Uuid, JobsSchedulerLocked) + Send + Sync;
-pub type JobToRunAsync = dyn FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> + Send + Sync;
+pub type JobToRunAsync = dyn FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+    + Send
+    + Sync;
+
+#[derive(Debug)]
+pub enum JobNotification {
+    Started,
+    Stopped,
+    Removed,
+}
+
+pub type OnJobNotification = dyn FnMut(Uuid, Uuid, JobNotification) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+    + Send
+    + Sync;
 
 fn nop(_uuid: Uuid, _jobs: JobsSchedulerLocked) {
     // Do nothing
 }
 
-fn nop_async(_uuid: Uuid, _jobs: JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> {
+fn nop_async(
+    _uuid: Uuid,
+    _jobs: JobsSchedulerLocked,
+) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> {
     Box::pin(async move {})
 }
 
 ///
 /// A schedulable Job
+#[derive(Clone)]
 pub struct JobLocked(pub(crate) Arc<RwLock<Box<dyn Job + Send + Sync>>>);
 
 pub enum JobType {
     CronJob,
     OneShot,
-    Repeated
+    Repeated,
 }
 
 pub trait Job {
@@ -48,6 +66,13 @@ pub trait Job {
     fn abort_join_handle(&mut self);
     fn stop(&self) -> bool;
     fn set_stopped(&mut self);
+    fn on_start_notification_add(&mut self, on_start: Box<OnJobNotification>) -> Uuid;
+    fn on_start_notification_remove(&mut self, id: Uuid) -> bool;
+    fn on_stop_notification_add(&mut self, on_stop: Box<OnJobNotification>) -> Uuid;
+    fn on_stop_notification_remove(&mut self, id: Uuid) -> bool;
+    fn on_removed_notification_add(&mut self, on_removed: Box<OnJobNotification>) -> Uuid;
+    fn on_removed_notification_remove(&mut self, id: Uuid) -> bool;
+    fn notify_on_removal(&mut self);
 }
 
 struct CronJob {
@@ -59,7 +84,10 @@ struct CronJob {
     pub count: u32,
     pub ran: bool,
     pub stopped: bool,
-    pub async_job: bool
+    pub async_job: bool,
+    pub on_start: HashMap<Uuid, Box<OnJobNotification>>,
+    pub on_stop: HashMap<Uuid, Box<OnJobNotification>>,
+    pub on_removed: HashMap<Uuid, Box<OnJobNotification>>,
 }
 
 impl Job for CronJob {
@@ -100,14 +128,37 @@ impl Job for CronJob {
     }
 
     fn run(&mut self, jobs: JobScheduler) {
-        if !self.async_job {
-            (self.run)(self.job_id, jobs);
-        } else {
-            let future = (self.run_async)(self.job_id, jobs);
+        for (uuid, on_start) in self.on_start.iter_mut() {
+            let future = (on_start)(self.job_id, *uuid, JobNotification::Started);
             tokio::task::spawn(async move {
                 future.await;
             });
         }
+
+        let job_id = self.job_id;
+        let on_stops = self
+            .on_stop
+            .iter_mut()
+            .map(|(uuid, on_stopped)| (on_stopped)(job_id, *uuid, JobNotification::Stopped))
+            .collect::<Vec<_>>();
+        if !self.async_job {
+            (self.run)(job_id, jobs);
+            for on_stop in on_stops {
+                tokio::task::spawn(async move {
+                    on_stop.await;
+                });
+            }
+        } else {
+            let future = (self.run_async)(job_id, jobs);
+            tokio::task::spawn(async move {
+                future.await;
+                for on_stop in on_stops {
+                    tokio::task::spawn(async move {
+                        on_stop.await;
+                    });
+                }
+            });
+        }
     }
 
     fn job_type(&self) -> &JobType {
@@ -133,6 +184,48 @@ impl Job for CronJob {
     fn set_stopped(&mut self) {
         self.stopped = true;
     }
+
+    fn on_start_notification_add(&mut self, on_start: Box<OnJobNotification>) -> Uuid {
+        let uuid = Uuid::new_v4();
+        self.on_start.insert(uuid, on_start);
+        uuid
+    }
+
+    fn on_start_notification_remove(&mut self, id: Uuid) -> bool {
+        self.on_start.remove(&id).is_some()
+    }
+
+    fn on_stop_notification_add(&mut self, on_stop: Box<OnJobNotification>) -> Uuid {
+        let uuid = Uuid::new_v4();
+        self.on_stop.insert(uuid, on_stop);
+        uuid
+    }
+
+    fn on_stop_notification_remove(&mut self, id: Uuid) -> bool {
+        self.on_stop.remove(&id).is_some()
+    }
+
+    fn on_removed_notification_add(&mut self, on_removed: Box<OnJobNotification>) -> Uuid {
+        let uuid = Uuid::new_v4();
+        self.on_removed.insert(uuid, on_removed);
+        uuid
+    }
+
+    fn on_removed_notification_remove(&mut self, id: Uuid) -> bool {
+        self.on_removed.remove(&id).is_some()
+    }
+
+    fn notify_on_removal(&mut self) {
+        let job_id = self.job_id;
+        self.on_removed
+            .iter_mut()
+            .map(|(uuid, on_removed)| (on_removed)(job_id, *uuid, JobNotification::Removed))
+            .for_each(|v| {
+                tokio::task::spawn(async move {
+                    v.await;
+                });
+            });
+    }
 }
 
 struct NonCronJob {
@@ -145,7 +238,10 @@ struct NonCronJob {
     pub count: u32,
     pub job_type: JobType,
     pub stopped: bool,
-    pub async_job: bool
+    pub async_job: bool,
+    pub on_start: HashMap<Uuid, Box<OnJobNotification>>,
+    pub on_stop: HashMap<Uuid, Box<OnJobNotification>>,
+    pub on_removed: HashMap<Uuid, Box<OnJobNotification>>,
 }
 
 impl Job for NonCronJob {
@@ -166,7 +262,6 @@ impl Job for NonCronJob {
     }
 
     fn set_count(&mut self, count: u32) {
-
         self.count = count;
     }
 
@@ -230,10 +325,51 @@ impl Job for NonCronJob {
     fn set_stopped(&mut self) {
         self.stopped = true;
     }
+
+    fn on_start_notification_add(&mut self, on_start: Box<OnJobNotification>) -> Uuid {
+        let uuid = Uuid::new_v4();
+        self.on_start.insert(uuid, on_start);
+        uuid
+    }
+
+    fn on_start_notification_remove(&mut self, id: Uuid) -> bool {
+        self.on_start.remove(&id).is_some()
+    }
+
+    fn on_stop_notification_add(&mut self, on_stop: Box<OnJobNotification>) -> Uuid {
+        let uuid = Uuid::new_v4();
+        self.on_stop.insert(uuid, on_stop);
+        uuid
+    }
+
+    fn on_stop_notification_remove(&mut self, id: Uuid) -> bool {
+        self.on_stop.remove(&id).is_some()
+    }
+
+    fn on_removed_notification_add(&mut self, on_removed: Box<OnJobNotification>) -> Uuid {
+        let uuid = Uuid::new_v4();
+        self.on_removed.insert(uuid, on_removed);
+        uuid
+    }
+
+    fn on_removed_notification_remove(&mut self, id: Uuid) -> bool {
+        self.on_removed.remove(&id).is_some()
+    }
+
+    fn notify_on_removal(&mut self) {
+        let job_id = self.job_id;
+        self.on_removed
+            .iter_mut()
+            .map(|(uuid, on_removed)| (on_removed)(job_id, *uuid, JobNotification::Removed))
+            .for_each(|v| {
+                tokio::task::spawn(async move {
+                    v.await;
+                });
+            });
+    }
 }
 
 impl JobLocked {
-
     /// Create a new cron job.
     ///
     /// ```rust,ignore
@@ -262,7 +398,10 @@ impl JobLocked {
                 count: 0,
                 ran: false,
                 stopped: false,
-                async_job: false
+                async_job: false,
+                on_stop: HashMap::new(),
+                on_start: HashMap::new(),
+                on_removed: HashMap::new(),
             }))),
         })
     }
@@ -282,7 +421,9 @@ impl JobLocked {
     pub fn new_async<T>(schedule: &str, run: T) -> Result<Self, Box<dyn std::error::Error>>
     where
         T: 'static,
-        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> + Send + Sync,
+        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+            + Send
+            + Sync,
     {
         let schedule: Schedule = Schedule::from_str(schedule)?;
         Ok(Self {
@@ -295,23 +436,26 @@ impl JobLocked {
                 count: 0,
                 ran: false,
                 stopped: false,
-                async_job: true
+                async_job: true,
+                on_stop: HashMap::new(),
+                on_start: HashMap::new(),
+                on_removed: HashMap::new(),
             }))),
         })
     }
 
-   /// Create a new cron job.
-   ///
-   /// ```rust,ignore
-   /// let mut sched = JobScheduler::new();
-   /// // Run at second 0 of the 15th minute of the 6th, 8th, and 10th hour
-   /// // of any day in March and June that is a Friday of the year 2017.
-   /// let job = Job::new_cron_job("0 15 6,8,10 * Mar,Jun Fri 2017", |_uuid, _lock| {
-   ///             println!("{:?} Hi I ran", chrono::Utc::now());
-   ///         });
-   /// sched.add(job)
-   /// tokio::spawn(sched.start());
-   /// ```
+    /// Create a new cron job.
+    ///
+    /// ```rust,ignore
+    /// let mut sched = JobScheduler::new();
+    /// // Run at second 0 of the 15th minute of the 6th, 8th, and 10th hour
+    /// // of any day in March and June that is a Friday of the year 2017.
+    /// let job = Job::new_cron_job("0 15 6,8,10 * Mar,Jun Fri 2017", |_uuid, _lock| {
+    ///             println!("{:?} Hi I ran", chrono::Utc::now());
+    ///         });
+    /// sched.add(job)
+    /// tokio::spawn(sched.start());
+    /// ```
     pub fn new_cron_job<T>(schedule: &str, run: T) -> Result<Self, Box<dyn std::error::Error>>
     where
         T: 'static,
@@ -335,12 +479,19 @@ impl JobLocked {
     pub fn new_cron_job_async<T>(schedule: &str, run: T) -> Result<Self, Box<dyn std::error::Error>>
     where
         T: 'static,
-        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> + Send + Sync,
+        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+            + Send
+            + Sync,
     {
         JobLocked::new_async(schedule, run)
     }
 
-    fn make_one_shot_job(duration: Duration, run: Box<JobToRun>, run_async: Box<JobToRunAsync>, async_job: bool) -> Result<Self, Box<dyn std::error::Error>> {
+    fn make_one_shot_job(
+        duration: Duration,
+        run: Box<JobToRun>,
+        run_async: Box<JobToRunAsync>,
+        async_job: bool,
+    ) -> Result<Self, Box<dyn std::error::Error>> {
         let job = NonCronJob {
             run,
             run_async,
@@ -351,10 +502,14 @@ impl JobLocked {
             count: 0,
             job_type: JobType::OneShot,
             stopped: false,
-            async_job
+            async_job,
+            on_stop: HashMap::new(),
+            on_start: HashMap::new(),
+            on_removed: HashMap::new(),
         };
 
-        let job: Arc<RwLock<Box<dyn Job + Send + Sync + 'static>>> = Arc::new(RwLock::new(Box::new(job)));
+        let job: Arc<RwLock<Box<dyn Job + Send + Sync + 'static>>> =
+            Arc::new(RwLock::new(Box::new(job)));
         let job_for_trigger = job.clone();
 
         let jh = tokio::spawn(async move {
@@ -376,9 +531,7 @@ impl JobLocked {
             j.set_join_handle(Some(jh));
         }
 
-        Ok(Self {
-            0: job
-        })
+        Ok(Self { 0: job })
     }
 
     /// Create a new one shot job.
@@ -411,15 +564,25 @@ impl JobLocked {
     /// sched.add(job)
     /// tokio::spawn(sched.start());
     /// ```
-    pub fn new_one_shot_async<T>(duration: Duration, run: T) -> Result<Self, Box<dyn std::error::Error>>
-        where
-            T: 'static,
-            T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> + Send + Sync,
+    pub fn new_one_shot_async<T>(
+        duration: Duration,
+        run: T,
+    ) -> Result<Self, Box<dyn std::error::Error>>
+    where
+        T: 'static,
+        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+            + Send
+            + Sync,
     {
         JobLocked::make_one_shot_job(duration, Box::new(nop), Box::new(run), true)
     }
 
-    fn make_new_one_shot_at_an_instant(instant: std::time::Instant, run: Box<JobToRun>, run_async: Box<JobToRunAsync>, async_job: bool) -> Result<Self, Box<dyn std::error::Error>> {
+    fn make_new_one_shot_at_an_instant(
+        instant: std::time::Instant,
+        run: Box<JobToRun>,
+        run_async: Box<JobToRunAsync>,
+        async_job: bool,
+    ) -> Result<Self, Box<dyn std::error::Error>> {
         let job = NonCronJob {
             run,
             run_async,
@@ -430,10 +593,14 @@ impl JobLocked {
             count: 0,
             job_type: JobType::OneShot,
             stopped: false,
-            async_job
+            async_job,
+            on_stop: HashMap::new(),
+            on_start: HashMap::new(),
+            on_removed: HashMap::new(),
         };
 
-        let job: Arc<RwLock<Box<dyn Job + Send + Sync + 'static>>> = Arc::new(RwLock::new(Box::new(job)));
+        let job: Arc<RwLock<Box<dyn Job + Send + Sync + 'static>>> =
+            Arc::new(RwLock::new(Box::new(job)));
         let job_for_trigger = job.clone();
 
         let jh = tokio::spawn(async move {
@@ -455,9 +622,7 @@ impl JobLocked {
             j.set_join_handle(Some(jh));
         }
 
-        Ok(Self {
-            0: job
-        })
+        Ok(Self { 0: job })
     }
 
     /// Create a new one shot job that runs at an instant
@@ -470,12 +635,20 @@ impl JobLocked {
     /// sched.add(job)
     /// tokio::spawn(sched.start());
     /// ```
-    pub fn new_one_shot_at_instant<T>(instant: std::time::Instant, run: T) -> Result<Self, Box<dyn std::error::Error>>
+    pub fn new_one_shot_at_instant<T>(
+        instant: std::time::Instant,
+        run: T,
+    ) -> Result<Self, Box<dyn std::error::Error>>
     where
         T: 'static,
         T: FnMut(Uuid, JobsSchedulerLocked) + Send + Sync,
     {
-        JobLocked::make_new_one_shot_at_an_instant(instant, Box::new(run), Box::new(nop_async), false)
+        JobLocked::make_new_one_shot_at_an_instant(
+            instant,
+            Box::new(run),
+            Box::new(nop_async),
+            false,
+        )
     }
 
     /// Create a new async one shot job that runs at an instant
@@ -488,15 +661,25 @@ impl JobLocked {
     /// sched.add(job)
     /// tokio::spawn(sched.start());
     /// ```
-    pub fn new_one_shot_at_instant_async<T>(instant: std::time::Instant, run: T) -> Result<Self, Box<dyn std::error::Error>>
+    pub fn new_one_shot_at_instant_async<T>(
+        instant: std::time::Instant,
+        run: T,
+    ) -> Result<Self, Box<dyn std::error::Error>>
     where
         T: 'static,
-        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> + Send + Sync,
+        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+            + Send
+            + Sync,
     {
         JobLocked::make_new_one_shot_at_an_instant(instant, Box::new(nop), Box::new(run), true)
     }
 
-    fn make_new_repeated(duration: Duration, run: Box<JobToRun>, run_async: Box<JobToRunAsync>, async_job: bool) -> Result<Self, Box<dyn std::error::Error>> {
+    fn make_new_repeated(
+        duration: Duration,
+        run: Box<JobToRun>,
+        run_async: Box<JobToRunAsync>,
+        async_job: bool,
+    ) -> Result<Self, Box<dyn std::error::Error>> {
         let job = NonCronJob {
             run,
             run_async,
@@ -507,10 +690,14 @@ impl JobLocked {
             count: 0,
             job_type: JobType::Repeated,
             stopped: false,
-            async_job
+            async_job,
+            on_stop: HashMap::new(),
+            on_start: HashMap::new(),
+            on_removed: HashMap::new(),
         };
 
-        let job: Arc<RwLock<Box<dyn Job + Send + Sync + 'static>>> = Arc::new(RwLock::new(Box::new(job)));
+        let job: Arc<RwLock<Box<dyn Job + Send + Sync + 'static>>> =
+            Arc::new(RwLock::new(Box::new(job)));
         let job_for_trigger = job.clone();
 
         let jh = tokio::spawn(async move {
@@ -535,9 +722,7 @@ impl JobLocked {
             j.set_join_handle(Some(jh));
         }
 
-        Ok(Self {
-            0: job
-        })
+        Ok(Self { 0: job })
     }
 
     /// Create a new repeated job.
@@ -552,9 +737,9 @@ impl JobLocked {
     /// tokio::spawn(sched.start());
     /// ```
     pub fn new_repeated<T>(duration: Duration, run: T) -> Result<Self, Box<dyn std::error::Error>>
-        where
-            T: 'static,
-            T: FnMut(Uuid, JobsSchedulerLocked) + Send + Sync,
+    where
+        T: 'static,
+        T: FnMut(Uuid, JobsSchedulerLocked) + Send + Sync,
     {
         JobLocked::make_new_repeated(duration, Box::new(run), Box::new(nop_async), false)
     }
@@ -570,10 +755,15 @@ impl JobLocked {
     /// sched.add(job)
     /// tokio::spawn(sched.start());
     /// ```
-    pub fn new_repeated_async<T>(duration: Duration, run: T) -> Result<Self, Box<dyn std::error::Error>>
-        where
-            T: 'static,
-            T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> + Send + Sync,
+    pub fn new_repeated_async<T>(
+        duration: Duration,
+        run: T,
+    ) -> Result<Self, Box<dyn std::error::Error>>
+    where
+        T: 'static,
+        T: FnMut(Uuid, JobsSchedulerLocked) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>>
+            + Send
+            + Sync,
     {
         JobLocked::make_new_repeated(duration, Box::new(nop), Box::new(run), true)
     }
@@ -585,52 +775,52 @@ impl JobLocked {
         let now = Utc::now();
         {
             let s = self.0.write();
-            s.map(|mut s| {
-                match s.job_type() {
-                    JobType::CronJob => {
-                        if s.last_tick().is_none() {
-                            s.set_last_tick(Some(now));
-                            return false;
-                        }
-                        let last_tick = *s.last_tick().unwrap();
+            s.map(|mut s| match s.job_type() {
+                JobType::CronJob => {
+                    if s.last_tick().is_none() {
                         s.set_last_tick(Some(now));
-                        s.increment_count();
-                        let must_run = s.schedule().unwrap()
-                            .after(&last_tick)
-                            .take(1)
-                            .map(|na| {
-                                let now_to_next = now.cmp(&na);
-                                let last_to_next = last_tick.cmp(&na);
+                        return false;
+                    }
+                    let last_tick = *s.last_tick().unwrap();
+                    s.set_last_tick(Some(now));
+                    s.increment_count();
+                    let must_run = s
+                        .schedule()
+                        .unwrap()
+                        .after(&last_tick)
+                        .take(1)
+                        .map(|na| {
+                            let now_to_next = now.cmp(&na);
+                            let last_to_next = last_tick.cmp(&na);
 
-                                matches!(now_to_next, std::cmp::Ordering::Greater) &&
-                                    matches!(last_to_next, std::cmp::Ordering::Less)
-                            })
-                            .into_iter()
-                            .find(|_| true)
-                            .unwrap_or(false);
+                            matches!(now_to_next, std::cmp::Ordering::Greater)
+                                && matches!(last_to_next, std::cmp::Ordering::Less)
+                        })
+                        .into_iter()
+                        .find(|_| true)
+                        .unwrap_or(false);
 
-                        if !s.ran() && must_run {
-                            s.set_ran(true);
-                        }
-                        must_run
-                    },
-                    JobType::OneShot => {
-                        if s.last_tick().is_some() {
-                            s.set_last_tick(None);
-                            s.set_ran(true);
-                            true
-                        } else {
-                            false
-                        }
-                    },
-                    JobType::Repeated => {
-                        if s.last_tick().is_some() {
-                            s.set_last_tick(None);
-                            s.set_ran(true);
-                            true
-                        } else {
-                            false
-                        }
+                    if !s.ran() && must_run {
+                        s.set_ran(true);
+                    }
+                    must_run
+                }
+                JobType::OneShot => {
+                    if s.last_tick().is_some() {
+                        s.set_last_tick(None);
+                        s.set_ran(true);
+                        true
+                    } else {
+                        false
+                    }
+                }
+                JobType::Repeated => {
+                    if s.last_tick().is_some() {
+                        s.set_last_tick(None);
+                        s.set_ran(true);
+                        true
+                    } else {
+                        false
                     }
                 }
             })
@@ -645,4 +835,52 @@ impl JobLocked {
         let r = self.0.read().unwrap();
         r.job_id()
     }
+
+    ///
+    /// Run something when the task is started. Returns a UUID as handle for this notification. This
+    /// UUID needs to be used when you want to remove the notification handle using `on_start_notification_remove`.
+    pub fn on_start_notification_add(&mut self, on_start: Box<OnJobNotification>) -> Uuid {
+        let mut w = self.0.write().unwrap();
+        w.on_start_notification_add(on_start)
+    }
+
+    ///
+    /// Remove the notification when the task was started. Uses the same UUID that was returned by
+    /// `on_start_notification_add`
+    pub fn on_start_notification_remove(&mut self, id: Uuid) -> bool {
+        let mut w = self.0.write().unwrap();
+        w.on_start_notification_remove(id)
+    }
+
+    ///
+    /// Run something when the task is stopped. Returns a UUID as handle for this notification. This
+    /// UUID needs to be used when you want to remove the notification handle using `on_stop_notification_remove`.
+    pub fn on_stop_notification_add(&mut self, on_stop: Box<OnJobNotification>) -> Uuid {
+        let mut w = self.0.write().unwrap();
+        w.on_stop_notification_add(on_stop)
+    }
+
+    ///
+    /// Remove the notification when the task was stopped. Uses the same UUID that was returned by
+    /// `on_stop_notification_add`
+    pub fn on_stop_notification_remove(&mut self, id: Uuid) -> bool {
+        let mut w = self.0.write().unwrap();
+        w.on_stop_notification_remove(id)
+    }
+
+    ///
+    /// Run something when the task was removed. Returns a UUID as handle for this notification. This
+    /// UUID needs to be used when you want to remove the notification handle using `on_removed_notification_remove`.
+    pub fn on_removed_notification_add(&mut self, on_removed: Box<OnJobNotification>) -> Uuid {
+        let mut w = self.0.write().unwrap();
+        w.on_removed_notification_add(on_removed)
+    }
+
+    ///
+    /// Remove the notification when the task was removed. Uses the same UUID that was returned by
+    /// `on_removed_notification_add`
+    pub fn on_removed_notification_remove(&mut self, id: Uuid) -> bool {
+        let mut w = self.0.write().unwrap();
+        w.on_removed_notification_remove(id)
+    }
 }
